@@ -5,12 +5,10 @@ use vars qw($AUTOLOAD $VERSION);
 
 use Carp;
 use Config;
-use Graph::Directed;
-use Math::Bezier;
 use IPC::Run qw(run binary);
 
 # This is incremented every time there is a change to the API
-$VERSION = '2.01';
+$VERSION = '2.02';
 
 =head1 NAME
 
@@ -35,11 +33,10 @@ GraphViz - Interface to the GraphViz graphing tool
 
 =head1 DESCRIPTION
 
-This module provides an interface to layout and image generation of
-directed and undirected graphs in a variety of formats (PostScript,
-PNG, etc.) using the "dot", "neato" and "twopi" programs from the
-GraphViz project (http://www.graphviz.org/ or
-http://www.research.att.com/sw/tools/graphviz/).
+This module provides an interface to layout and image generation of directed
+and undirected graphs in a variety of formats (PostScript, PNG, etc.) using the
+"dot", "neato", "twopi", "circo" and "fdp"  programs from the GraphViz project
+(http://www.graphviz.org/ or http://www.research.att.com/sw/tools/graphviz/).
 
 =head2 What is a graph?
 
@@ -121,61 +118,203 @@ prize.
 
 This is the constructor. It accepts several attributes.
 
-The most two important attributes are 'layout' and 'directed'. The
-'layout' attribute determines which layout algorithm GraphViz.pm will
-use. Possible values are: 'dot' (the default GraphViz layout for
-directed graph layouts), 'neato' (for undirected graph layouts -
-spring model) or 'twopi' (for undirected graph layouts - circular).
+  my $g = GraphViz->new();
+  my $g = GraphViz->new(directed => 0);
+  my $g = GraphViz->new(layout => 'neato', ratio => 'compress');
+  my $g = GraphViz->new(rankdir  => 1);
+  my $g = GraphViz->new(width => 8.5, height => 11);
+  my $g = GraphViz->new(width => 30, height => 20,
+			pagewidth => 8.5, pageheight => 11);
+
+The most two important attributes are 'layout' and 'directed'.
+
+=over
+
+=item layout
+
+The 'layout' attribute determines which layout algorithm GraphViz.pm will
+use. Possible values are: 
+
+=over
+
+=item dot
+
+The default GraphViz layout for directed graph layouts
+
+=item neato
+
+For undirected graph layouts - spring model
+
+=item twopi
+
+For undirected graph layouts - radial
+
+=item circo
+
+For undirected graph layouts - circular
+
+=item fdp
+
+For undirected graph layouts - force directed spring model
+
+=back
+
+=item directed
 
 The 'directed' attribute, which defaults to 1 (true) specifies
 directed (edges have arrows) graphs. Setting this to zero produces
 undirected graphs (edges do not have arrows).
 
+=item rankdir
+
 Another attribute 'rankdir' controls the direction the nodes are linked
 together. If true it will do left->right linking rather than the
 default up-down linking.
 
+=item width, height
+
 The 'width' and 'height' attributes control the size of the bounding
 box of the drawing in inches. This is more useful for PostScript
 output as for raster graphic (such as PNG) the pixel dimensions
 can not be set, although there are generally 96 pixels per inch.
 
+=item pagewidth, pageheight
+
 The 'pagewidth' and 'pageheight' attributes set the PostScript
 pagination size in inches. That is, if the image is larger than the
 page then the resulting PostScript image is a sequence of pages that
 can be tiled or assembled into a mosaic of the full image. (This only
 works for PostScript output).
 
-  my $g = GraphViz->new();
-  my $g = GraphViz->new(directed => 0);
-  my $g = GraphViz->new(layout => 'neato');
-  my $g = GraphViz->new(rankdir  => 1);
-  my $g = GraphViz->new(width => 8.5, height => 11);
-  my $g = GraphViz->new(width => 30, height => 20,
-			pagewidth => 8.5, pageheight => 11);
+
+=item concentrate
 
 The 'concentrate' attribute controls enables an edge merging technique
 to reduce clutter in dense layouts of directed graphs. The default is
 not to merge edges.
 
+=item random_start
+
 For undirected graphs, the 'random_start' attribute requests an
 initial random placement for the graph, which may give a better
 result. The default is not random.
 
+=item epsilon
+
 For undirected graphs, the 'epsilon' attribute decides how long the
 graph solver tries before finding a graph layout. Lower numbers allow
 the solver to fun longer and potentially give a better layout. Larger
 values can decrease the running time but with a reduction in layout
 quality. The default is 0.1.
 
+=item overlap
+
+The 'overlap' option allows you to set layout behavior for graph nodes
+that overlap.  (From GraphViz documentation:)
+
+Determines if and how node overlaps should be removed.
+
+
+=over
+
+=item true
+
+(the default) overlaps are retained. 
+
+=item scale
+
+overlaps are removed by uniformly scaling in x and y. 
+
+=item false 
+
+If the value converts to "false", node overlaps are removed by a Voronoi-based technique.
+
+=item scalexy
+
+x and y are separately scaled to remove overlaps.
+
+=item orthoxy, orthxy
+
+If the value is "orthoxy" or "orthoyx", overlaps are moved by optimizing two
+constraint problems, one for the x axis and one for the y. The suffix indicates
+which axis is processed first.
+
+B<NOTE>: The methods related to "orthoxy" and "orthoyx" are still evolving. The
+semantics of these may change, or these methods may disappear altogether. 
+
+=item compress
+
+If the value is "compress", the layout will be scaled down as much as possible
+without introducing any overlaps.
+
+=back
+
+Except for the Voronoi method, all of these transforms preserve the orthogonal
+ordering of the original layout. That is, if the x coordinates of two nodes are
+originally the same, they will remain the same, and if the x coordinate of one
+node is originally less than the x coordinate of another, this relation will
+still hold in the transformed layout. The similar properties hold for the y
+coordinates.
+
+=item no_overlap
+
 The 'no_overlap' overlap option, if set, tells the graph solver to not
-overlap the nodes.
+overlap the nodes.  Deprecated,  Use 'overlap' => 'false'.
+
+=item ratio
+
+The 'ratio' option sets the aspect ratio (drawing height/drawing width) for the
+drawing. Note that this is adjusted before the size attribute constraints are
+enforced.  Default value is C<fill>.
+
+=over
+
+=item numeric
+
+If ratio is numeric, it is taken as the desired aspect ratio. Then, if the
+actual aspect ratio is less than the desired ratio, the drawing height is
+scaled up to achieve the desired ratio; if the actual ratio is greater than
+that desired ratio, the drawing width is scaled up.
+
+=item fill
+
+If ratio = C<fill> and the size attribute is set, node positions are scaled,
+separately in both x and y, so that the final drawing exactly fills the
+specified size.
+
+=item compress
+
+If ratio = C<compress> and the size attribute is set, dot attempts to compress
+the initial layout to fit in the given size. This achieves a tighter packing of
+nodes but reduces the balance and symmetry. This feature only works in dot.
+
+=item expand
+
+If ratio = C<expand> the size attribute is set, and both the width and the
+height of the graph are less than the value in size, node positions are scaled
+uniformly until at least one dimension fits size exactly. Note that this is
+distinct from using size as the desired size, as here the drawing is expanded
+before edges are generated and all node and text sizes remain unchanged.
+
+=item auto
+
+If ratio = C<auto> the page attribute is set and the graph cannot be drawn on a
+single page, then size is set to an ``ideal'' value. In particular, the size in
+a given dimension will be the smallest integral multiple of the page size in
+that dimension which is at least half the current size. The two dimensions are
+then scaled independently to the new size. This feature only works in dot. 
+
+=back
+
+=item bgcolor
 
 The 'bgcolor' option sets the background colour. A colour value may be
 "h,s,v" (hue, saturation, brightness) floating point numbers between 0
 and 1, or an X11 color name such as 'white', 'black', 'red', 'green',
 'blue', 'yellow', 'magenta', 'cyan', or 'burlywood'.
 
+=item node,edge,graph
+
 The 'node', 'edge' and 'graph' attributes allow you to specify global
 node, edge and graph attributes (in addition to those controlled by
 the special attributes described above). The value should be a hash
@@ -184,6 +323,8 @@ to make all nodes box-shaped (unless explicity given another shape):
 
   my $g = GraphViz->new(node => {shape => 'box'});
 
+=back
+
 =cut
 
 
@@ -203,7 +344,6 @@ sub new {
   $self->{NODES} = {};
   $self->{NODELIST} = [];
   $self->{EDGES} = [];
-  $self->{GRAPH} = Graph::Directed->new();
 
   if (exists $config->{directed}) {
       $self->{DIRECTED} = $config->{directed};
@@ -237,7 +377,11 @@ sub new {
 
   $self->{SORT} = $config->{sort} if (exists $config->{sort});
 
-  $self->{NO_OVERLAP} = $config->{no_overlap} if (exists $config->{no_overlap});
+  $self->{OVERLAP} = $config->{overlap} if (exists $config->{overlap});
+  # no_overlap overrides overlap setting.
+  $self->{OVERLAP} = 'false' if (exists $config->{no_overlap});
+
+  $self->{RATIO} = $config->{ratio} || 'fill';
 
   # Global node, edge and graph attributes
   $self->{NODE_ATTRS} = $config->{node} if (exists $config->{node});
@@ -408,7 +552,9 @@ sub add_node {
       $node->{label} = $node->{name};
     }
   } else {
-   $node->{label} =~ s#([|<>\[\]{}"])#\\$1#g unless $node->{shape} && $node->{shape} eq 'record';
+    $node->{label} =~ s#([|<>\[\]{}"])#\\$1#g unless $node->{shape} &&
+      ($node->{shape} eq 'record' || ($node->{label} =~ /^<</ && $node->{shape} eq
+				      'plaintext'));
   }
 
   delete $node->{cluster}
@@ -436,11 +582,6 @@ sub add_node {
   }
 
   $self->{CODES}->{$node->{_code}} = $node->{name};
-  $self->{GRAPH}->add_vertex($node->{name});
-
-  foreach my $key (keys %$node) {
-    $self->{GRAPH}->set_attribute($key, $node->{name}, $node->{$key});
-  }
 
   # Add the node to the nodelist, which contains the names of
   # all the nodes in the order that they were inserted (but only
@@ -581,31 +722,19 @@ sub add_edge {
     return;
   }
 
+  my $from = $edge->{from};
+  my $to = $edge->{to};
+  $self->add_node($from) unless exists $self->{NODES}->{$from};
+  $self->add_node($to) unless exists $self->{NODES}->{$to};
+
   push @{$self->{EDGES}}, $edge; # should remove!
-
-  # The Graph module is strict about the nodes existing before we add
-  # edges, but graphviz is not, so we check. Any nodes that don't
-  # exist, we create with all the default attributes.
-  unless ($self->{GRAPH}->has_vertex($edge->{from})) {
-    $self->add_node($edge->{from});
-  }
-  unless ($self->{GRAPH}->has_vertex($edge->{to})) {
-    $self->add_node($edge->{to});
-  }
-
-  $self->{GRAPH}->add_edge($edge->{from} => $edge->{to});
-
-  foreach my $key (keys %$edge) {
-    $self->{GRAPH}->set_attribute($key, $edge->{from}, $edge->{to}, $edge->{$key});
-  }
-
 }
 
 
 =head2 as_canon, as_text, as_gif etc. methods
 
 There are a number of methods which generate input for dot / neato /
-twopi or output the graph in a variety of formats.
+twopi / circo / fdp or output the graph in a variety of formats.
 
 Note that if you pass a filename, the data is written to that
 filename. If you pass a filehandle, the data will be streamed to the
@@ -638,7 +767,7 @@ does not lay out the graph. This is mostly useful for debugging.
 
 =item as_canon
 
-The as_canon method returns the canonical dot / neato / twopi file
+The as_canon method returns the canonical dot / neato / twopi / circo / fdp  file
 which corresponds to the graph. It does not layout the graph - every
 other as_* method does.
 
@@ -659,7 +788,7 @@ other as_* method does.
 =item as_text
 
 The as_text method returns text which is a layed-out dot / neato /
-twopi format file.
+twopi / circo / fdp format file.
 
   print $g->as_text;
 
@@ -743,17 +872,24 @@ Returns a string which contains a layed-out Windows BMP-format file.
 
   print $g->as_wbmp;
 
-=item as_cmap
+=item as_cmap  (deprecated)
 
 Returns a string which contains a layed-out HTML client-side image map
-format file.
+format file.   Use as_cmpax instead.
 
   print $g->as_cmap;
 
-=item as_ismap
+=item as_cmapx
+
+Returns a string which contains a layed-out HTML HTML/X client-side image map
+format file.
+
+  print $g->as_cmapx;
+
+=item as_ismap (deprecated)
 
 Returns a string which contains a layed-out old-style server-side
-image map format file.
+image map format file.  Use as_imap instead.
 
   print $g->as_ismap;
 
@@ -795,6 +931,13 @@ Returns a string which contains a layed-out SVG-format file.
 
   print $g->as_svg;
 
+=item as_svgz
+
+Returns a string which contains a layed-out SVG-format file
+that is compressed.
+
+  print $g->as_svgz;
+
 =item as_plain
 
 Returns a string which contains a layed-out simple-format file.
@@ -822,7 +965,7 @@ sub AUTOLOAD {
     $name = "as_dot";
   }
 
-  if ($name =~ /^as_(ps|hpgl|pcl|mif|pic|gd|gd2|gif|jpeg|png|wbmp|cmap|ismap|imap|vrml|vtx|mp|fig|svg|dot|canon|plain)$/) {
+  if ($name =~ /^as_(ps|hpgl|pcl|mif|pic|gd|gd2|gif|jpeg|png|wbmp|cmapx?|ismap|imap|vrml|vtx|mp|fig|svgz?|dot|canon|plain)$/) {
     my $data = $self->_as_generic('-T' . $1, $self->_as_debug, $output);
     return $data;
   }
@@ -831,71 +974,6 @@ sub AUTOLOAD {
 }
 
 
-# Undocumented feature: return a Graph object
-sub as_graph {
-  my($self, $conf) = @_;
-  my $graph = $self->{GRAPH};
-
-  return $self->_parse_dot($self->_as_debug);
-}
-
-
-sub _parse_dot {
-  my($self, $dot) = @_;
-  my $graph = $self->{GRAPH};
-
-  my $out;
-  my $program = $self->{LAYOUT};
-
-  run [$program, '-Tplain'], \$dot, \$out;
-
-  my($aspect, $bbw, $bbh);
-
-  foreach my $line (split /\n/, $out) {
-#    print "# $line\n";
-
-    my($type, @values) = split /\s+/, $line;
-    if ($type eq 'graph') {
-      ($aspect, $bbw, $bbh) = @values;
-    } elsif ($type eq 'node') {
-      my($node, $x, $y, $w, $h) = @values;
-      $x /= $bbw;
-      $y /= $bbh;
-      $w /= $bbw;
-      $h /= $bbh;
-      $node = $self->{CODES}->{$node};
-#      print "#  $node  ($x, $y) x ($w, $h)\n";
-      $graph->set_attribute('x', $node, $x);
-      $graph->set_attribute('y', $node, $y);
-      $graph->set_attribute('w', $node, $w);
-      $graph->set_attribute('h', $node, $h);
-    } elsif ($type eq 'edge') {
-      my($from, $to, $n, @points) = @values;
-
-      $from = $self->{CODES}->{$from};
-        $to = $self->{CODES}->{$to};
-
-      @points = splice(@points, 0, $n * 2);
-
-      my @newpoints;
-
-      while (@points) {
-	my ($x, $y) = splice(@points, 0, 2);
-	$x /= $bbw;
-	$y /= $bbh;
-	push @newpoints, $x, $y;
-      }
-
-      my $bezier = Math::Bezier->new(@newpoints);
-#      print "#  $from->$to: @newpoints\n";
-      $graph->set_attribute('bezier', $from, $to, $bezier);
-    }
-#    next unless $type eq 'node';
-  }
-
-  return $graph;
-}
-
 # Return the main dot text
 sub as_debug {
   my $self = shift;
@@ -915,8 +993,11 @@ sub _as_debug {
   $dot .= "\trankdir=LR;\n" if $self->{RANK_DIR};
 
   # the size of the graph
-  $dot .= "\tsize=\"" . $self->{WIDTH} . "," . $self->{HEIGHT} ."\";\n\tratio=fill\n" if $self->{WIDTH} && $self->{HEIGHT};
+  $dot .= "\tsize=\"" . $self->{WIDTH} . "," . $self->{HEIGHT} ."\";\n" if $self->{WIDTH} && $self->{HEIGHT};
   $dot .= "\tpage=\"" . $self->{PAGEWIDTH} . "," . $self->{PAGEHEIGHT} ."\";\n" if $self->{PAGEWIDTH} && $self->{PAGEHEIGHT};
+  
+  # Ratio setting
+  $dot .= "\tratio=\"" . $self->{RATIO} . "\";\n";
 
   # edge merging
   $dot .= "\tconcentrate=true;\n" if $self->{CONCENTRATE};
@@ -927,8 +1008,8 @@ sub _as_debug {
   # random start
   $dot .= "\tstart=rand;\n" if $self->{RANDOM_START};
 
-  # no_overlap
-  $dot .= "\toverlap=false;\n" if $self->{NO_OVERLAP};
+  # overlap
+  $dot .= "\toverlap=\"" . $self->{OVERLAP} . "\";\n" if $self->{OVERLAP};
 
   # color, bgcolor
   $dot .= "\tbgcolor=\"" . $self->{BGCOLOR} . "\";\n" if $self->{BGCOLOR};
@@ -1042,7 +1123,7 @@ sub _as_debug {
 }
 
 
-# Call dot / neato / twopi with the input text and any parameters
+# Call dot / neato / twopi / circo / fdp with the input text and any parameters
 
 sub _as_generic {
   my($self, $type, $dot, $output) = @_;
@@ -1070,7 +1151,7 @@ sub _as_generic {
 }
 
 
-# Quote a node/edge name using dot / neato / twopi's quoting rules
+# Quote a node/edge name using dot / neato / circo / fdp / twopi's quoting rules
 
 sub _quote_name {
   my($self, $name) = @_;
@@ -1096,7 +1177,7 @@ sub _quote_name {
 }
 
 
-# Return the attributes of a node or edge as a dot / neato / twopi attribute
+# Return the attributes of a node or edge as a dot / neato / circo / fdp / twopi attribute
 # string
 
 sub _attributes {
@@ -1110,7 +1191,7 @@ sub _attributes {
 
     my $value = $thing->{$key};
     $value =~ s|"|\"|g;
-    $value = '"' . $value . '"';
+    $value = '"' . $value . '"' unless ($key eq 'label' && $value =~ /^<</);
     $value =~ s|\n|\\n|g;
 
     $value = '""' if not defined $value;
